//
// Fl_Native_File_Chooser_DEFAULT.cxx -- FLTK native OS file chooser widget
//
// Copyright 2004 by Greg Ercolano.
// March 2005 - wrapper around Fl_File_Chooser by Nathan Vander Wilt
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
#include <FL/Fl_Native_File_Chooser.H>

// CTOR
Fl_Native_File_Chooser::Fl_Native_File_Chooser() {
    Ctor(BROWSE_FILE);
}

// CTOR
Fl_Native_File_Chooser::Fl_Native_File_Chooser(Fl_Native_File_Chooser::Type val) {
    Ctor(val);
}

// CTOR
void Fl_Native_File_Chooser::Ctor(Fl_Native_File_Chooser::Type type) {
    _errmsg          = NULL;
    file_chooser=new Fl_File_Chooser(NULL,NULL,type_fl_file(type),NULL);
}

// DTOR
Fl_Native_File_Chooser::~Fl_Native_File_Chooser() {
    if ( _errmsg ) free(_errmsg);
    if ( file_chooser ) delete file_chooser;
}

// PRIVATE: SET ERROR MESSAGE
void Fl_Native_File_Chooser::errmsg(const char *msg) {
    if ( _errmsg ) free(_errmsg);
    _errmsg = strdup(msg);
}

// PRIVATE: translate Native types to Fl_File_Chooser types
int Fl_Native_File_Chooser::type_fl_file(Type val) {
 switch (val) {
  case BROWSE_FILE:
   return (Fl_File_Chooser::SINGLE);
  case BROWSE_DIRECTORY:                  // adds create...but should that be in BROWSE_SAVE_DIRECTORY ?
   return ( Fl_File_Chooser::SINGLE | Fl_File_Chooser::DIRECTORY | Fl_File_Chooser::CREATE );
  case BROWSE_MULTI_FILE:
   return (Fl_File_Chooser::MULTI);
  case BROWSE_MULTI_DIRECTORY:
   return ( Fl_File_Chooser::DIRECTORY | Fl_File_Chooser::MULTI );
  case BROWSE_SAVE_FILE:
   return ( Fl_File_Chooser::SINGLE | Fl_File_Chooser::CREATE );
  case BROWSE_NOSAVE_DIRECTORY:
   return (Fl_File_Chooser::DIRECTORY | Fl_File_Chooser::SINGLE );
  default:
   return (Fl_File_Chooser::SINGLE);
 }
}

void Fl_Native_File_Chooser::type(Fl_Native_File_Chooser::Type val) {
 if (file_chooser) file_chooser->type(type_fl_file(val));
}

// Show chooser, blocks until done.
// RETURNS:
//    0 - user picked a file
//    1 - user cancelled
//   -1 - failed; errmsg() has reason
//
int Fl_Native_File_Chooser::show() {
 if (!file_chooser) { errmsg("Couldn't create default FLTK File Chooser."); return -1; }
 file_chooser->show();
 while (file_chooser->shown()) Fl::wait();  // necessary to follow documented blocking behavior
 if ( file_chooser->count() ) return 0;
 else return 1;                           // ! TBD/XXX How to tell exit status? Is this to do with the (undocumented?) callback of Fl_File_Chooser?
}

// RETURN ERROR MESSAGE
const char *Fl_Native_File_Chooser::errmsg() {
    return(_errmsg?_errmsg:"No error");
}

// GET FILENAME
const char* Fl_Native_File_Chooser::filename() const {
    if ( file_chooser && file_chooser->count()>0 ) return(file_chooser->value());
}

// GET FILENAME FROM LIST OF FILENAMES
const char* Fl_Native_File_Chooser::filename(int i) const {
    if ( file_chooser && i < file_chooser->count() ) return(file_chooser->value(i));
    return("");
}

// SET TITLE
//     Can be NULL if no title desired.
//
void Fl_Native_File_Chooser::title(const char *val) {
    if (file_chooser) file_chooser->label(val);
}

// GET TITLE
//    Can return NULL if none set.
//
const char *Fl_Native_File_Chooser::title() const {
    if (file_chooser) return(file_chooser->label());
    return NULL;
}

// SET FILTER
//     Can be NULL if no filter needed
//
void Fl_Native_File_Chooser::filter(const char *val) {
    if(val) puts(val);
    if (file_chooser) file_chooser->filter(val);
}

// GET FILTER
//    Can return NULL if none set.
//
const char *Fl_Native_File_Chooser::filter() const {
    if (file_chooser) return file_chooser->filter();
    return NULL;
}

// GET TOTAL FILENAMES CHOSEN
int Fl_Native_File_Chooser::size() {
    if (file_chooser) return(file_chooser->count());
    return 0;
}
int Fl_Native_File_Chooser::total_filenames() const {        // TBD/XXX: used by main.cxx, but undocumented.
    if (file_chooser) return(file_chooser->count());
    return 0;
}


// PRESET PATHNAME
//     Can be NULL if no preset is desired.
//
void Fl_Native_File_Chooser::preset_dirname(const char *val) {
    if (file_chooser) file_chooser->directory(val);
}

// GET PRESET PATHNAME
//    Can return NULL if none set.
//
const char *Fl_Native_File_Chooser::preset_dirname() const {
    if (file_chooser) return(file_chooser->directory());
    return NULL;
}


